@@ -1,20 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2007 Jeffrey Stedfast
+ *  Copyright (C) 2000-2008 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -27,6 +28,18 @@
 
 #include "gmime-filter-strip.h"
 
+
+/**
+ * SECTION: gmime-filter-strip
+ * @title: GMimeFilterStrip
+ * @short_description: Strip trailing whitespace from the end of lines
+ * @see_also: #GMimeFilter
+ *
+ * A #GMimeFilter used for stripping trailing whitespace from the end
+ * of lines.
+ **/
+
+
 static void g_mime_filter_strip_class_init (GMimeFilterStripClass *klass);
 static void g_mime_filter_strip_init (GMimeFilterStrip *filter, GMimeFilterStripClass *klass);
 static void g_mime_filter_strip_finalize (GObject *object);
@@ -165,7 +178,7 @@ filter_reset (GMimeFilter *filter)
  * Returns a new strip filter.
  **/
 GMimeFilter *
-g_mime_filter_strip_new ()
+g_mime_filter_strip_new (void)
 {
 	GMimeFilterStrip *new;
 	
